@@ -9,9 +9,17 @@ import inspect
 import json
 import os
 import re
+import codecs
 from functools import wraps
 
-__version__ = '1.0.3'
+try:
+    import yaml
+except ImportError:  # pragma: no cover
+    _have_yaml = False
+else:
+    _have_yaml = True
+
+__version__ = '1.1.2'
 
 # These attributes will not conflict with any real python attribute
 # They are added to the decorated test method and processed later
@@ -20,6 +28,7 @@ __version__ = '1.0.3'
 DATA_ATTR = '%values'      # store the data the test must run with
 FILE_ATTR = '%file_path'   # store the path to JSON file
 UNPACK_ATTR = '%unpack'    # remember that we have to unpack values
+index_len = 5              # default max length of case index
 
 
 try:
@@ -51,9 +60,21 @@ def data(*values):
 
     Should be added to methods of instances of ``unittest.TestCase``.
 
+    """
+    global index_len
+    index_len = len(str(len(values)))
+    return idata(values)
+
+
+def idata(iterable):
+    """
+    Method decorator to add to your test methods.
+
+    Should be added to methods of instances of ``unittest.TestCase``.
+
     """
     def wrapper(func):
-        setattr(func, DATA_ATTR, values)
+        setattr(func, DATA_ATTR, iterable)
         return func
     return wrapper
 
@@ -101,15 +122,17 @@ def mk_test_name(name, value, index=0):
     only of trivial values.
     """
 
+    # Add zeros before index to keep order
+    index = "{0:0{1}}".format(index + 1, index_len)
     if not is_trivial(value):
-        return "{0}_{1}".format(name, index + 1)
+        return "{0}_{1}".format(name, index)
     try:
         value = str(value)
     except UnicodeEncodeError:
         # fallback for python2
         value = value.encode('ascii', 'backslashreplace')
-    test_name = "{0}_{1}_{2}".format(name, index + 1, value)
-    return re.sub('\W|^(?=\d)', '_', test_name)
+    test_name = "{0}_{1}_{2}".format(name, index, value)
+    return re.sub(r'\W|^(?=\d)', '_', test_name)
 
 
 def feed_data(func, new_name, *args, **kwargs):
@@ -121,6 +144,7 @@ def feed_data(func, new_name, *args, **kwargs):
     def wrapper(self):
         return func(self, *args, **kwargs)
     wrapper.__name__ = new_name
+    wrapper.__wrapped__ = func
     # Try to call format on the docstring
     if func.__doc__:
         try:
@@ -152,25 +176,56 @@ def process_file_data(cls, name, func, file_attr):
     cls_path = os.path.abspath(inspect.getsourcefile(cls))
     data_file_path = os.path.join(os.path.dirname(cls_path), file_attr)
 
-    def _raise_ve(*args):  # pylint: disable-msg=W0613
-        raise ValueError("%s does not exist" % file_attr)
+    def create_error_func(message):  # pylint: disable-msg=W0613
+        def func(*args):
+            raise ValueError(message % file_attr)
+        return func
 
-    if os.path.exists(data_file_path) is False:
+    # If file does not exist, provide an error function instead
+    if not os.path.exists(data_file_path):
         test_name = mk_test_name(name, "error")
-        add_test(cls, test_name, _raise_ve, None)
-    else:
-        data = json.loads(open(data_file_path).read())
-        for i, elem in enumerate(data):
-            if isinstance(data, dict):
-                key, value = elem, data[elem]
-                test_name = mk_test_name(name, key, i)
-            elif isinstance(data, list):
-                value = elem
-                test_name = mk_test_name(name, value, i)
-            if isinstance(value, dict):
-                add_test(cls, test_name, func, **value)
-            else:
-                add_test(cls, test_name, func, value)
+        add_test(cls, test_name, create_error_func("%s does not exist"), None)
+        return
+
+    _is_yaml_file = data_file_path.endswith((".yml", ".yaml"))
+
+    # Don't have YAML but want to use YAML file.
+    if _is_yaml_file and not _have_yaml:
+        test_name = mk_test_name(name, "error")
+        add_test(
+            cls,
+            test_name,
+            create_error_func("%s is a YAML file, please install PyYAML"),
+            None
+        )
+        return
+
+    with codecs.open(data_file_path, 'r', 'utf-8') as f:
+        # Load the data from YAML or JSON
+        if _is_yaml_file:
+            data = yaml.safe_load(f)
+        else:
+            data = json.load(f)
+
+    _add_tests_from_data(cls, name, func, data)
+
+
+def _add_tests_from_data(cls, name, func, data):
+    """
+    Add tests from data loaded from the data file into the class
+
+    """
+    for i, elem in enumerate(data):
+        if isinstance(data, dict):
+            key, value = elem, data[elem]
+            test_name = mk_test_name(name, key, i)
+        elif isinstance(data, list):
+            value = elem
+            test_name = mk_test_name(name, value, i)
+        if isinstance(value, dict):
+            add_test(cls, test_name, func, **value)
+        else:
+            add_test(cls, test_name, func, value)
 
 
 def ddt(cls):
